/*******************************************************************************
***
*** Filename         : net2tty.c
*** Purpose          : TCP socket <-> RS232 bridge
*** Author           : Matt J. Gumbley
*** Last updated     : 22/05/99
*** Notes            : Connects to a port at a host, opens a tty, sets the baud
***                    rate, and sends any data coming in over the socket to 
***                    the tty, and sends any data received from the tty to 
***                    the socket.
***
********************************************************************************
***
*** Modification Record
***
*******************************************************************************/

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include "global.h"
#include "asy.h"

static char *defdevice = "/dev/ttyS0";
#define DEVICEMAX 20
static char device[DEVICEMAX];

static int defbps=B9600;
static int bps;
#define BAUDMAX 10
static char baudstr[BAUDMAX];

static int verbose = FALSE;
static int animate = FALSE;

#define HOSTMAX 30
static char hostname[HOSTMAX];
static char *defhostname = "localhost";
static int port;
static int portfd;
static int sockfd;
static int quit=0;

static char animchars[]="\\-/|";

void sighandler(int signal)
{
  quit=1;
}

static void banner()
{
  printf("net2tty - a simple network <-> RS232 bridge\n");
  printf("(C) 1999 Matt J. Gumbley, matt@gumbley.demon.co.uk\n");
  printf("http://www.gumbley.demon.co.uk/\n\n");
}

static void usage()
{
  banner();
  printf("Syntax:\n");
  printf("  net2tty [options]\n");
  printf("Options:\n");
  printf("  -b<speed>  Set port speed. <speed> is 300 to 115200 [9600]\n");
  printf("  -d<device> Set tty device. <device> is like /dev/ttyS0.\n");
  printf("             Default is %s.\n", defdevice);
  printf("  -h<host>   Set host to connect to; name or IP address\n");
  printf("             Default is %s.\n", defhostname);
  printf("  -p<port>   Set port to connect to; numeric only\n");
  printf("  -v         Enable verbose operation\n");
  printf("  -a         Animate a spinner on stderr when data is received.\n");
  printf("             Useful when redirecting to a file to see that\n");
  printf("             something is happening.\n");
  printf("\n");
  printf("Options -b, -h and -p are mandatory.\n");
  printf("Example: net2tty -b9600 -d/dev/ttyS1 -hlocalhost -p25\n");
  printf("This will route your SMTP connection over the serial line.\n");
  exit(1);
}

static void spinner()
{
static int c=0;
  fprintf(stderr,"%c  \r",animchars[c++]);
  c&=3;
}

void foo()
{
unsigned char buf;
int readdata;
  while (!quit) {
    readdata = read(portfd, &buf, 1);
    printf("read %d bytes from port\n", readdata);
  }
}



void main(int argc, char *argv[])
{
int i;
char *p;
struct sockaddr_in sin;
struct hostent *phe;
fd_set arfds, rfds;
unsigned char buf;
int hd;
char portanim='\\', sockanim='\\';
int portanidx=0, sockanidx=0;

  strcpy(baudstr,"9600");
  strcpy(device, defdevice);
  strcpy(hostname, defhostname);
  hostname[0]='\0';
  port='0';

  for (i=1; i< argc; i++) {
    if (argv[i][0] == '-') {
      p=&argv[i][2];
      switch (argv[i][1]) {
        case 'a':
          animate = TRUE;
          break;
        case 'b':
          strncpy(baudstr, p, BAUDMAX);
          if (strcmp(p, "300")==0)
            bps=B300;
          else if (strcmp(p, "600")==0)
            bps=B600;
          else if (strcmp(p, "1200")==0)
            bps=B1200;
          else if (strcmp(p, "2400")==0)
            bps=B2400;
          else if (strcmp(p, "4800")==0)
            bps=B4800;
          else if (strcmp(p, "9600")==0)
            bps=B9600;
          else if (strcmp(p, "19200")==0)
            bps=B19200;
          else if (strcmp(p, "38400")==0)
            bps=B38400;
          else if (strcmp(p, "57600")==0)
            bps=B57600;
          else if (strcmp(p, "115200")==0)
            bps=B115200;
          else
            usage();
          break;
        case 'd':
          strncpy(device, p, DEVICEMAX);
          break;
        case 'v':
          verbose = TRUE;
          break;
        case 'h':
          strncpy(hostname, p, HOSTMAX);
          break;
        case 'p':
          sscanf(p, "%d", &port);
          break;
        default:
          usage();
      }
    }
    else {
      printf("option encountered without leading - ?\n");
      usage();
    }
  }

  if (port == 0 || hostname[0]=='\0')
    usage();

  /* Trap CTRL-C */
  signal(SIGINT, &sighandler);

  /* Open port */
  if ((portfd=asy_open(device, bps)) == -1) {
    printf("port %s open failed\n", device);
    exit(-1);
  }

  if (verbose) {
    printf("port: %s at %s baud, descriptor is %d\n", device, baudstr, portfd);
  }

  /* Open network connection */
  if (verbose) {
    printf("opening socket...\n");
  }
  if ((sockfd = socket(AF_INET, SOCK_STREAM, 6 /* tcp */)) == -1) {
    printf("socket open failed\n");
    exit(-1);
  }
  if (verbose) {
    printf("socket descriptor is %d\n", sockfd);
  }
  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons(port);
  if (verbose) {
    printf("resolving any hostname...\n");
  }
  if ((phe=gethostbyname(hostname)))
    memcpy(&sin.sin_addr, phe->h_addr, phe->h_length);
  else if ((sin.sin_addr.s_addr=inet_addr(hostname)) == INADDR_NONE) {
    printf("host %s is invalid; can't get host entry\n", hostname);
    exit(-1);
  }
  if (verbose) {
    printf("connecting to %s:%d...\n", hostname, port);
  }
  if (connect(sockfd, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
    printf("can't connect to %s:%d : %s\n", hostname, port, strerror(errno));
    exit(-1);
  }

  /* Set up read sets */
  FD_ZERO(&arfds);
  FD_SET(portfd, &arfds);
  FD_SET(sockfd, &arfds);
  hd = portfd > sockfd ? portfd : sockfd;
  hd++;

  if (verbose) {
    printf("entering main loop...\n");
  }
/*  foo();
goto enderd;*/

  /* Transfer data */
  while (!quit) {
    memcpy(&rfds, &arfds, sizeof(rfds));

    if (select(hd, &rfds, NULL, NULL, NULL) < 0) {
      if (errno == EINTR) /* Just a signal */
        continue;
      printf("select failed: %s\n", strerror(errno));
      exit(-1);
    }
    if (FD_ISSET(portfd, &rfds)) {
      portanim = animchars[portanidx++];
      portanidx &= 3;

      read(portfd, &buf, 1);
      write(sockfd, &buf, 1);
    }
    if (FD_ISSET(sockfd, &rfds)) {
      sockanim = animchars[sockanidx++];
      sockanidx &= 3;

      read(sockfd, &buf, 1);
      write(portfd, &buf, 1);
    }
    if (verbose) {
      fprintf(stderr, "\r  PORT %c SOCKET %c \r", portanim, sockanim);
    }
  }
enderd:
  if (verbose) {
    printf("Closing down\n");
  }
  /* Goodnight Vienna */
  shutdown(sockfd, 2);
  asy_close(portfd);
}

